# -*- coding: utf-8 -*-
"""
The deepdata modules included utilities to load datasets, including public and private datasets, these datasets were fundamental in data analysis
as well as models. Models developed by engineers could be validated by these datasets
"""



from .base import load_classification_dataset
from .base import describe_dataset
from .base import load_phm
from .base import load_dna
from .base import load_usps
from .base import load_satimage
from .base import load_letter
from .base import load_letter_2_class
from .base import load_arrhythmia
from .base import load_cardio
from .base import load_mnist
from .base import load_musk
from .base import load_optdigits
from .base import load_pendigits
from .base import load_pima
from .base import load_satellite
from .base import load_shuttle
from .base import load_speech
from .base import load_vowels
from .base import load_phm_outlier_10pct
from .base import load_phm_outlier_20pct
from .base import load_phm_outlier_30pct
from .base import load_phm_outlier_40pct



# FIXME: put actual code here
def example():
    pass

def hello():
    print('Hello, I am deepdata.')

